//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import "AppDelegate.h"
#import "TokenManager.h"
#import "UserTokens.h"
#import "Logger.h"
#import "ViewController.h"
#import <YounifyConnectSDK/YounifyConnectSDK.h>

@interface AppDelegate ()

@end

@implementation AppDelegate


- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary<UIApplicationLaunchOptionsKey, id> *)launchOptions
{
    // Override point for customization after application launch.

    TokenManager *tokenManager = [TokenManager shared];
    UserTokens *userTokens = tokenManager.userTokens;

    YCSConnectOptions *options = [[YCSConnectOptions alloc] init];
    options.logLevel = YCSLogLevelWarning;
    options.logDelegate = [Logger shared];
    options.tokenDelegate = tokenManager;
    options.key = @"_YOUR_SDK_KEY_HERE_"; //TODO: Set your sdk key.
    options.accessToken = userTokens.accessToken;
    options.refreshToken = userTokens.refreshToken;

    [[YCSConnect shared] configure:options];

    return YES;
}


- (BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options
{
    //TODO: Remove the following block and scheme registration in Info.plist for production apps!
    //The following block allows configuring the Younify Connect SDK from a url in order to facilitate testing.
    //The url format is ycssampleapp://configure/?key=<sdk key>&at=<user's access token>&rt=<user's refresh token>.
    if ([url.scheme isEqualToString:@"ycssampleapp"])
    {
        NSURLComponents *components = [NSURLComponents componentsWithURL:url resolvingAgainstBaseURL:YES];
        if ([components.host isEqualToString:@"configure"])
        {
            NSArray<NSURLQueryItem *> *queryItems = components.queryItems;
            NSPredicate *keyPredicate = [NSPredicate predicateWithFormat:@"name=%@", @"key"];
            NSPredicate *atPredicate = [NSPredicate predicateWithFormat:@"name=%@", @"at"];
            NSPredicate *rtPredicate = [NSPredicate predicateWithFormat:@"name=%@", @"rt"];
            NSPredicate *extraPredicate = [NSPredicate predicateWithFormat:@"name=%@", @"extra"];
            NSURLQueryItem *keyQueryItem = [[queryItems filteredArrayUsingPredicate:keyPredicate] firstObject];
            NSURLQueryItem *atQueryItem = [[queryItems filteredArrayUsingPredicate:atPredicate] firstObject];
            NSURLQueryItem *rtQueryItem = [[queryItems filteredArrayUsingPredicate:rtPredicate] firstObject];
            NSURLQueryItem *extraQueryItem = [[queryItems filteredArrayUsingPredicate:extraPredicate] firstObject];

            NSDictionary<NSString *, id> *extra = nil;
            if (extraQueryItem && extraQueryItem.value && extraQueryItem.value.length)
            {
                NSData *extraJSONData = [extraQueryItem.value dataUsingEncoding:NSUTF8StringEncoding];
                id extraJSONObject = [NSJSONSerialization JSONObjectWithData:extraJSONData options:NSJSONReadingFragmentsAllowed error:nil];
                if (extraJSONObject && [extraJSONObject isKindOfClass:[NSDictionary<NSString *, id> class]])
                    extra = extraJSONObject;
            }

            if (keyQueryItem && atQueryItem && rtQueryItem)
            {
                YCSConnectOptions *options = [[YCSConnectOptions alloc] init];
                options.logLevel = YCSLogLevelWarning;
                options.logDelegate = [Logger shared];
                options.tokenDelegate = nil; //XXX: Because the key isn't being persisted, prevent token renewals from being persisted as well.
                options.key = keyQueryItem.value;
                options.accessToken = atQueryItem.value;
                options.refreshToken = rtQueryItem.value;
                options.extra = extra;
                [[YCSConnect shared] configure:options];
                [(ViewController *)self.window.rootViewController refresh];
                return YES;
            }
        }
    }

    return NO;
}


@end
