//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import <UIKit/UIKit.h>
#import "CategoryContentCell.h"

static CGFloat const CategoryContentImageWidth = 149;
static CGFloat const CategoryContentImageHeight = 84;

@interface CategoryContentCell()

@property (nonatomic, strong) UILabel *nameLabel;
@property (nonatomic, strong) UIImageView *contentImageView;
@property (nonatomic, strong) UIImageView *serviceImageView;

@end

@implementation CategoryContentCell

- (instancetype)initWithFrame:(CGRect)rect
{
    if (self = [super initWithFrame:rect])
        [self setupUI];

    return self;
}

- (void)setupUI
{
    self.contentImageView = [[UIImageView alloc] init];
    self.contentImageView.contentMode = UIViewContentModeScaleAspectFill;
    [self.contentView addSubview:self.contentImageView];
    self.contentImageView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.contentImageView.widthAnchor constraintEqualToConstant:CategoryContentImageWidth],
        [self.contentImageView.heightAnchor constraintEqualToConstant:CategoryContentImageHeight],
        [self.contentImageView.topAnchor constraintEqualToAnchor:self.contentView.topAnchor constant:0],
        [self.contentImageView.leftAnchor constraintEqualToAnchor:self.contentView.leftAnchor constant:0]]];

    self.serviceImageView = [[UIImageView alloc] init];
    self.serviceImageView.contentMode = UIViewContentModeScaleAspectFill;
    self.serviceImageView.clipsToBounds = YES;
    [self.contentView addSubview:self.serviceImageView];
    self.serviceImageView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.serviceImageView.widthAnchor constraintEqualToConstant:36],
        [self.serviceImageView.heightAnchor constraintEqualToConstant:20],
        [self.serviceImageView.rightAnchor constraintEqualToAnchor:self.contentImageView.rightAnchor constant:0],
        [self.serviceImageView.bottomAnchor constraintEqualToAnchor:self.contentImageView.bottomAnchor constant:0]]];

    self.nameLabel = [[UILabel alloc] init];
    self.nameLabel.textColor = [UIColor whiteColor];
    self.nameLabel.font = [UIFont systemFontOfSize:14];
    [self.contentView addSubview:self.nameLabel];
    self.nameLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.nameLabel.heightAnchor constraintEqualToConstant:20],
        [self.nameLabel.topAnchor constraintEqualToAnchor:self.contentImageView.bottomAnchor constant:0],
        [self.nameLabel.leftAnchor constraintEqualToAnchor:self.contentImageView.leftAnchor constant:0],
        [self.nameLabel.rightAnchor constraintEqualToAnchor:self.contentImageView.rightAnchor constant:0]]];

    self.contentView.backgroundColor = [UIColor clearColor];
    self.backgroundColor = [UIColor clearColor];
}

- (void)updateWithContentAndService:(YCSStreamingContent *)item service:(YCSStreamingService *)service
{
    self.nameLabel.text = item.title;

    [[YCSConnect shared] fetchImage:item.smallThumbnailUrl cancellationTicket:nil completionHandler:^(NSData *data, NSError *error)
    {
        [self.contentImageView setImage:[UIImage imageWithData:data]];
    }];

    if (service != nil)
        [[YCSConnect shared] fetchImage:service.overlayThumbnailUrl cancellationTicket:nil completionHandler:^(NSData *data, NSError *error)
        {
            [self.serviceImageView setImage:[UIImage imageWithData:data]];
        }];
}

@end

