//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "ContentCell.h"
#import "Resources.h"

static CGFloat const ContentThumbWidthRequest = 86;
static CGFloat const ContentThumbHeightRequest = 48;

@interface ContentCell()

@property (nonatomic, strong) UILabel *nameLabel;
@property (nonatomic, strong) UIImageView *contentImageView;
@property (nonatomic, strong) UIImageView *serviceImageView;

@end

@implementation ContentCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier;
{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier])
        [self setupUI];

    return self;
}

- (void)setupUI
{
    self.contentImageView = [[UIImageView alloc] init];
    self.contentImageView.contentMode = UIViewContentModeScaleAspectFill;
    self.serviceImageView.clipsToBounds = YES;
    [self.contentView addSubview:self.contentImageView];
    self.contentImageView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.contentImageView.widthAnchor constraintEqualToConstant:ContentThumbWidthRequest],
        [self.contentImageView.heightAnchor constraintEqualToConstant:ContentThumbHeightRequest],
        [self.contentImageView.topAnchor constraintEqualToAnchor:self.contentView.topAnchor constant:2],
        [self.contentImageView.leftAnchor constraintEqualToAnchor:self.contentView.leftAnchor constant:0]]];

    self.serviceImageView = [[UIImageView alloc] init];
    self.serviceImageView.contentMode = UIViewContentModeScaleAspectFill;
    self.serviceImageView.clipsToBounds = YES;
    [self.contentView addSubview:self.serviceImageView];
    self.serviceImageView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.serviceImageView.widthAnchor constraintEqualToConstant:36],
        [self.serviceImageView.heightAnchor constraintEqualToConstant:20],
        [self.serviceImageView.rightAnchor constraintEqualToAnchor:self.contentImageView.rightAnchor constant:0],
        [self.serviceImageView.bottomAnchor constraintEqualToAnchor:self.contentImageView.bottomAnchor constant:0]]];

    self.nameLabel = [[UILabel alloc] init];
    self.nameLabel.textColor = [UIColor whiteColor];
    self.nameLabel.font = [UIFont systemFontOfSize:14];
    [self.contentView addSubview:self.nameLabel];
    self.nameLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.nameLabel.topAnchor constraintEqualToAnchor:self.contentImageView.topAnchor constant:0],
        [self.nameLabel.leftAnchor constraintEqualToAnchor:self.contentImageView.rightAnchor constant:4],
        [self.nameLabel.rightAnchor constraintEqualToAnchor:self.contentView.rightAnchor constant:0],
        [self.nameLabel.bottomAnchor constraintEqualToAnchor:self.contentImageView.bottomAnchor constant:0]]];

    self.contentView.backgroundColor = [UIColor clearColor];
    self.backgroundColor = [UIColor clearColor];

    UIView *backgroundView = [[UIView alloc] init];
    backgroundView.backgroundColor = [Resources mainColor];
    self.selectedBackgroundView = backgroundView;
}

- (void)updateWithContentAndService:(YCSStreamingContent *)item service:(YCSStreamingService *)service
{
    self.nameLabel.text = item.title;
    [[YCSConnect shared] fetchImage:item.smallThumbnailUrl cancellationTicket:nil completionHandler:^(NSData *data, NSError *error)
    {
        [self.contentImageView setImage:[UIImage imageWithData:data]];
    }];

    if (service != nil)
    {
        NSString *overlayThumbnailUrl = service.overlayThumbnailUrl;
        if ((overlayThumbnailUrl != nil) && [overlayThumbnailUrl isKindOfClass:[NSString class]] && (overlayThumbnailUrl.length != 0))
            [[YCSConnect shared] fetchImage:overlayThumbnailUrl cancellationTicket:nil completionHandler:^(NSData *data, NSError *error)
            {
                [self.serviceImageView setImage:[UIImage imageWithData:data]];
            }];
    }
}

@end
