//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import "ContentDetailsViewController.h"
#import "Resources.h"

static CGFloat const LabelsSpacing = 15;

@interface ContentDetailsViewController ()

@property (nonatomic, strong) YCSStreamingContent *content;
@property (nonatomic, strong) YCSStreamingContentDetails *contentDetails;
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIStackView *scrollViewContainer;
@property (nonatomic, strong) UIImageView *contentImageView;
@property (nonatomic, strong) UILabel *nameLabel;
@property (nonatomic, strong) UILabel *titleLabel;
@property (nonatomic, strong) UILabel *overviewLabel;
@property (nonatomic, strong) UILabel *durationLabel;
@property (nonatomic, strong) UILabel *estimatedDurationLabel;
@property (nonatomic, strong) UILabel *airDateLabel;
@property (nonatomic, strong) UILabel *seriesLabel;
@property (nonatomic, strong) UILabel *seasonLabel;
@property (nonatomic, strong) UILabel *episodeLabel;
@property (nonatomic, strong) UILabel *releaseYearLabel;
@property (nonatomic, strong) UILabel *contentRatingLabel;
@property (nonatomic, strong) UITextView *watchNowUrlLabel;
@property (nonatomic, strong) UILabel *itemIDLabel;
@property (nonatomic, strong) UILabel *tmdbIDLabel;
@property (nonatomic, strong) UILabel *tmdbSeriesIDLabel;
@property (nonatomic, strong) UILabel *pathLabel;
@property (nonatomic, strong) UIActivityIndicatorView *activityIndicator;

@property (nonatomic, strong) YCSConnect *ycsConnect;

@end

@implementation ContentDetailsViewController

- (instancetype)initWithContent:(YCSStreamingContent *)content;
{
    if (self = [super init])
        self.content = content;

    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.ycsConnect = [YCSConnect shared];
    [self setupUI];
    [self.activityIndicator startAnimating];
    [self loadContentDetailsWithCompletionHandler:^
    {
        [self.activityIndicator stopAnimating];
        [self updateUI];
    }];
}

- (void)setupUI
{
    self.scrollView = [[UIScrollView alloc] init];
    [self.view addSubview:self.scrollView];
    self.scrollView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.scrollView.topAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.topAnchor constant:20],
        [self.scrollView.leftAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.leftAnchor constant:20],
        [self.scrollView.rightAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.rightAnchor constant:-20],
        [self.scrollView.bottomAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.bottomAnchor constant:0]]];

    self.scrollViewContainer = [[UIStackView alloc] init];
    self.scrollViewContainer.axis = UILayoutConstraintAxisVertical;
    self.scrollViewContainer.spacing = LabelsSpacing;
    [self.scrollView addSubview:self.scrollViewContainer];
    self.scrollViewContainer.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.scrollViewContainer.widthAnchor constraintEqualToAnchor:self.scrollView.widthAnchor],
        [self.scrollViewContainer.leadingAnchor constraintEqualToAnchor:self.scrollView.leadingAnchor constant:0],
        [self.scrollViewContainer.trailingAnchor constraintEqualToAnchor:self.scrollView.trailingAnchor constant:0],
        [self.scrollViewContainer.topAnchor constraintEqualToAnchor:self.scrollView.topAnchor constant:0],
        [self.scrollViewContainer.bottomAnchor constraintEqualToAnchor:self.scrollView.bottomAnchor constant:0]]];

    self.contentImageView = [[UIImageView alloc] init];
    self.contentImageView.contentMode = UIViewContentModeScaleAspectFit;
    [self.scrollViewContainer addArrangedSubview:self.contentImageView];
    self.contentImageView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.contentImageView.heightAnchor constraintEqualToAnchor:self.view.heightAnchor multiplier:0.3],
        [self.contentImageView.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.contentImageView.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.nameLabel = [[UILabel alloc] init];
    self.nameLabel.textColor = [UIColor whiteColor];
    self.nameLabel.numberOfLines = 0;
    self.nameLabel.lineBreakMode = NSLineBreakByWordWrapping;
    [self.scrollViewContainer addArrangedSubview:self.nameLabel];
    self.nameLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.nameLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.nameLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.titleLabel = [[UILabel alloc] init];
    self.titleLabel.textColor = [UIColor whiteColor];
    self.titleLabel.font = [UIFont systemFontOfSize:14];
    self.titleLabel.numberOfLines = 0;
    self.titleLabel.lineBreakMode = NSLineBreakByWordWrapping;
    [self.scrollViewContainer addArrangedSubview:self.titleLabel];
    self.titleLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.titleLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.titleLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.overviewLabel = [[UILabel alloc] init];
    self.overviewLabel.textColor = [UIColor whiteColor];
    self.overviewLabel.font = [UIFont systemFontOfSize:14];
    self.overviewLabel.numberOfLines = 0;
    self.overviewLabel.lineBreakMode = NSLineBreakByWordWrapping;
        [self.scrollViewContainer addArrangedSubview:self.overviewLabel];
    self.overviewLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.overviewLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.overviewLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.durationLabel = [[UILabel alloc] init];
    self.durationLabel.textColor = [UIColor whiteColor];
    self.durationLabel.font = [UIFont systemFontOfSize:14];
    [self.scrollViewContainer addArrangedSubview:self.durationLabel];
    self.durationLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.durationLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.durationLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.estimatedDurationLabel = [[UILabel alloc] init];
    self.estimatedDurationLabel.textColor = [UIColor whiteColor];
    self.estimatedDurationLabel.font = [UIFont systemFontOfSize:14];
    [self.scrollViewContainer addArrangedSubview:self.estimatedDurationLabel];
    self.estimatedDurationLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.estimatedDurationLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.estimatedDurationLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.airDateLabel = [[UILabel alloc] init];
    self.airDateLabel.textColor = [UIColor whiteColor];
    self.airDateLabel.font = [UIFont systemFontOfSize:14];
    [self.scrollViewContainer addArrangedSubview:self.airDateLabel];
    self.airDateLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.airDateLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.airDateLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.seriesLabel = [[UILabel alloc] init];
    self.seriesLabel.textColor = [UIColor whiteColor];
    self.seriesLabel.font = [UIFont systemFontOfSize:14];
    self.seriesLabel.numberOfLines = 0;
    self.seriesLabel.lineBreakMode = NSLineBreakByWordWrapping;
    [self.scrollViewContainer addArrangedSubview:self.seriesLabel];
    self.seriesLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.seriesLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.seriesLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.seasonLabel = [[UILabel alloc] init];
    self.seasonLabel.textColor = [UIColor whiteColor];
    self.seasonLabel.font = [UIFont systemFontOfSize:14];
    [self.scrollViewContainer addArrangedSubview:self.seasonLabel];
    self.seasonLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.seasonLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.seasonLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.episodeLabel = [[UILabel alloc] init];
    self.episodeLabel.textColor = [UIColor whiteColor];
    self.episodeLabel.font = [UIFont systemFontOfSize:14];
    [self.scrollViewContainer addArrangedSubview:self.episodeLabel];
    self.episodeLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.episodeLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.episodeLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.releaseYearLabel = [[UILabel alloc] init];
    self.releaseYearLabel.textColor = [UIColor whiteColor];
    self.releaseYearLabel.font = [UIFont systemFontOfSize:14];
    [self.scrollViewContainer addArrangedSubview:self.releaseYearLabel];
    self.releaseYearLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.releaseYearLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.releaseYearLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.contentRatingLabel = [[UILabel alloc] init];
    self.contentRatingLabel.textColor = [UIColor whiteColor];
    self.contentRatingLabel.font = [UIFont systemFontOfSize:14];
    [self.scrollViewContainer addArrangedSubview:self.contentRatingLabel];
    self.contentRatingLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.contentRatingLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.contentRatingLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.watchNowUrlLabel = [[UITextView alloc] init];
    self.watchNowUrlLabel.textContainer.maximumNumberOfLines = 1;
    self.watchNowUrlLabel.textContainer.lineBreakMode = NSLineBreakByTruncatingTail;
    self.watchNowUrlLabel.layoutManager.usesFontLeading = FALSE;
    self.watchNowUrlLabel.contentInset = UIEdgeInsetsZero;
    self.watchNowUrlLabel.textContainerInset = UIEdgeInsetsZero;
    self.watchNowUrlLabel.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    self.watchNowUrlLabel.textContainer.lineFragmentPadding = 0;
    self.watchNowUrlLabel.backgroundColor = [UIColor clearColor];
    self.watchNowUrlLabel.textColor = [UIColor whiteColor];
    self.watchNowUrlLabel.font = [UIFont systemFontOfSize:14];
    self.watchNowUrlLabel.editable = NO;
    [self.scrollViewContainer addArrangedSubview:self.watchNowUrlLabel];
    self.watchNowUrlLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.watchNowUrlLabel.heightAnchor constraintEqualToConstant:20],
        [self.watchNowUrlLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.watchNowUrlLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.itemIDLabel = [[UILabel alloc] init];
    self.itemIDLabel.textColor = [UIColor whiteColor];
    self.itemIDLabel.font = [UIFont systemFontOfSize:14];
    self.itemIDLabel.numberOfLines = 0;
    self.itemIDLabel.lineBreakMode = NSLineBreakByWordWrapping;
    [self.scrollViewContainer addArrangedSubview:self.itemIDLabel];
    self.itemIDLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.itemIDLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.itemIDLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.tmdbIDLabel = [[UILabel alloc] init];
    self.tmdbIDLabel.textColor = [UIColor whiteColor];
    self.tmdbIDLabel.font = [UIFont systemFontOfSize:14];
    self.tmdbIDLabel.numberOfLines = 0;
    self.tmdbIDLabel.lineBreakMode = NSLineBreakByWordWrapping;
    [self.scrollViewContainer addArrangedSubview:self.tmdbIDLabel];
    self.tmdbIDLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.tmdbIDLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.tmdbIDLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.tmdbSeriesIDLabel = [[UILabel alloc] init];
    self.tmdbSeriesIDLabel.textColor = [UIColor whiteColor];
    self.tmdbSeriesIDLabel.font = [UIFont systemFontOfSize:14];
    self.tmdbSeriesIDLabel.numberOfLines = 0;
    self.tmdbSeriesIDLabel.lineBreakMode = NSLineBreakByWordWrapping;
    [self.scrollViewContainer addArrangedSubview:self.tmdbSeriesIDLabel];
    self.tmdbSeriesIDLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.tmdbSeriesIDLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.tmdbSeriesIDLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.pathLabel = [[UILabel alloc] init];
    self.pathLabel.textColor = [UIColor whiteColor];
    self.pathLabel.font = [UIFont systemFontOfSize:14];
    self.pathLabel.numberOfLines = 0;
    self.pathLabel.lineBreakMode = NSLineBreakByWordWrapping;
    [self.scrollViewContainer addArrangedSubview:self.pathLabel];
    self.pathLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.pathLabel.leftAnchor constraintEqualToAnchor:self.scrollViewContainer.leftAnchor constant:0],
        [self.pathLabel.rightAnchor constraintEqualToAnchor:self.scrollViewContainer.rightAnchor constant:0]]];

    self.activityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleMedium];
    self.activityIndicator.hidesWhenStopped = TRUE;
    self.activityIndicator.color = [Resources mainColor];
    self.activityIndicator.center = self.view.center;
    [self.view addSubview:self.activityIndicator];
    self.activityIndicator.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.activityIndicator.centerXAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.centerXAnchor constant:0],
        [self.activityIndicator.centerYAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.centerYAnchor constant:0],
    ]];

    self.view.backgroundColor = [UIColor blackColor];
}

- (void)updateUI
{
    self.nameLabel.text = self.content.name;
    self.titleLabel.text = self.content.title;
    self.overviewLabel.text = [NSString stringWithFormat:@"%@",self.content.overview];
    self.durationLabel.text = [NSString stringWithFormat:@"Duration: %lld", self.content.duration];
    self.airDateLabel.text = [NSString stringWithFormat:@"Air Date: %@", self.content.airDate];
    self.seriesLabel.text = [NSString stringWithFormat:@"Series: %@", self.content.series];
    self.seasonLabel.text = [NSString stringWithFormat:@"Season: %@", self.content.season];
    self.episodeLabel.text = [NSString stringWithFormat:@"Episode: %@", self.content.episode];
    self.releaseYearLabel.text = [NSString stringWithFormat:@"Release Year: %@", self.content.releaseYear];
    self.contentRatingLabel.text = [NSString stringWithFormat:@"Content Rating: %@", self.content.contentRating];

    if ((self.content.watchNowUrl != nil) && ![self.content.watchNowUrl isKindOfClass:[NSNull class]] && (self.content.watchNowUrl.length > 0)) 
    {
        NSString *watchNowText = [NSString stringWithFormat:@"Watch Now Url: %@", self.content.watchNowUrl];
        NSMutableAttributedString *attributeStr = [[NSMutableAttributedString alloc]initWithString:watchNowText attributes:nil];
        [attributeStr addAttribute:NSLinkAttributeName value:self.content.watchNowUrl range:[watchNowText rangeOfString:self.content.watchNowUrl]];
        [self.watchNowUrlLabel setAttributedText:attributeStr];
        self.watchNowUrlLabel.textColor = [UIColor whiteColor];
        self.watchNowUrlLabel.linkTextAttributes = @{NSForegroundColorAttributeName:[UIColor colorWithRed:82.0/255.0 green:83.0/255.0 blue:232.0/255.0 alpha:1.0]};
    }

    self.itemIDLabel.text = [NSString stringWithFormat:@"Item ID: %@", self.content.itemID];
    self.tmdbIDLabel.text = [NSString stringWithFormat:@"TMDB ID: %@", self.content.tmdbID];
    self.tmdbSeriesIDLabel.text = [NSString stringWithFormat:@"TMDB Series ID: %@", self.content.tmdbSeriesID];
    self.pathLabel.text = [NSString stringWithFormat:@"Path: %@", [self.content.path componentsJoinedByString:@" | "]];

    if (self.contentDetails)
    {
        self.estimatedDurationLabel.text = [NSString stringWithFormat:@"Estimated Duration: %lld", self.contentDetails.estimatedDuration];
    }

    [self.ycsConnect fetchImage:self.content.largeThumbnailUrl cancellationTicket:nil completionHandler:^(NSData *data, NSError *error)
    {
        [self.contentImageView setImage:[UIImage imageWithData:data]];
    }];
}

- (void)loadContentDetailsWithCompletionHandler:(void (^)(void))completionHandler
{
    [self.ycsConnect fetchContentDetails:self.content cancellationTicket:nil completionHandler:^(YCSStreamingContent *content, YCSStreamingContentDetails *details, NSError *error)
    {
        self.contentDetails = details;

        if (completionHandler)
            completionHandler();
    }];
}

@end
