//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import "ServiceCell.h"
#import "CategoriesViewController.h"
#import "Resources.h"

@interface ServiceCell()

@property (nonatomic, strong) UIView *boxView;
@property (nonatomic, strong) UILabel *serviceName;
@property (nonatomic, strong) UILabel *username;
@property (nonatomic, strong) UILabel *profileName;
@property (nonatomic, strong) UIImageView *serviceImageView;
@property (nonatomic, strong) UIImageView *brokenServiceImageView;
@property (nonatomic, strong) UIButton *browseButton;
@property (nonatomic, strong) UIButton *logoutButton;
@property (nonatomic, strong) NSLayoutConstraint *serviceNameHeightConstraint;

@end

@implementation ServiceCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier;
{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier])
        [self setupUI];

    return self;
}

- (void)setupUI
{
    self.serviceImageView = [[UIImageView alloc] init];
    [self.contentView addSubview:self.serviceImageView];
    self.serviceImageView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.serviceImageView.widthAnchor constraintEqualToAnchor:self.contentView.heightAnchor constant:-4],
        [self.serviceImageView.heightAnchor constraintEqualToAnchor:self.contentView.heightAnchor constant:-4],
        [self.serviceImageView.topAnchor constraintEqualToAnchor:self.contentView.topAnchor constant:2],
        [self.serviceImageView.leftAnchor constraintEqualToAnchor:self.contentView.leftAnchor constant:12]]];

    self.brokenServiceImageView = [[UIImageView alloc] init];
    self.brokenServiceImageView.image = [UIImage systemImageNamed:@"exclamationmark.triangle.fill"];
    self.brokenServiceImageView.tintColor = [UIColor redColor];
    self.brokenServiceImageView.hidden = TRUE;
    [self.contentView addSubview:self.brokenServiceImageView];
    self.brokenServiceImageView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.brokenServiceImageView.widthAnchor constraintEqualToConstant:20],
        [self.brokenServiceImageView.heightAnchor constraintEqualToConstant:20],
        [self.brokenServiceImageView.rightAnchor constraintEqualToAnchor:self.serviceImageView.rightAnchor constant:3],
        [self.brokenServiceImageView.bottomAnchor constraintEqualToAnchor:self.serviceImageView.bottomAnchor constant:3]]];

    self.serviceName = [[UILabel alloc] init];
    self.serviceName.textColor = [UIColor whiteColor];
    [self.contentView addSubview:self.serviceName];
    self.serviceName.translatesAutoresizingMaskIntoConstraints = NO;
    self.serviceNameHeightConstraint = [self.serviceName.heightAnchor constraintEqualToAnchor:self.serviceImageView.heightAnchor multiplier:0.33];
    [NSLayoutConstraint activateConstraints:@[
        [self.serviceName.topAnchor constraintEqualToAnchor:self.serviceImageView.topAnchor constant:0],
        [self.serviceName.leftAnchor constraintEqualToAnchor:self.serviceImageView.rightAnchor constant:5],
        [self.serviceName.rightAnchor constraintEqualToAnchor:self.contentView.rightAnchor constant:0],
        self.serviceNameHeightConstraint]];

    self.username = [[UILabel alloc] init];
    self.username.textColor = [UIColor whiteColor];
    self.username.font = [UIFont systemFontOfSize:10];
    self.username.hidden = TRUE;
    [self.contentView addSubview:self.username];
    self.username.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.username.topAnchor constraintEqualToAnchor:self.serviceName.bottomAnchor constant:0],
        [self.username.leftAnchor constraintEqualToAnchor:self.serviceImageView.rightAnchor constant:5],
        [self.username.rightAnchor constraintEqualToAnchor:self.contentView.rightAnchor constant:0],
        [self.username.heightAnchor constraintEqualToAnchor:self.contentView.heightAnchor multiplier:0.33]]];

    self.profileName = [[UILabel alloc] init];
    self.profileName.textColor = [UIColor whiteColor];
    self.profileName.font = [UIFont systemFontOfSize:10];
    self.profileName.hidden = TRUE;
    [self.contentView addSubview:self.profileName];
    self.profileName.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.profileName.topAnchor constraintEqualToAnchor:self.username.bottomAnchor constant:0],
        [self.profileName.leftAnchor constraintEqualToAnchor:self.serviceImageView.rightAnchor constant:5],
        [self.profileName.rightAnchor constraintEqualToAnchor:self.contentView.rightAnchor constant:0],
        [self.profileName.heightAnchor constraintEqualToAnchor:self.contentView.heightAnchor multiplier:0.33]]];

    self.logoutButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.logoutButton setTitleColor:[UIColor colorWithRed:32.0/255.0 green:32.0/255.0 blue:32.0/255.0 alpha:1.0] forState:UIControlStateNormal];
    self.logoutButton.backgroundColor = [Resources mainColor];
    self.logoutButton.layer.borderColor = [UIColor clearColor].CGColor;
    self.logoutButton.layer.borderWidth = 0.0f;
    self.logoutButton.layer.cornerRadius = 10.0f;
    self.logoutButton.titleLabel.font = [UIFont systemFontOfSize:12];
    [self.logoutButton addTarget:self action:@selector(logoutButtonClicked:) forControlEvents:UIControlEventTouchUpInside];
    [self.logoutButton setTitle:@"logout" forState:UIControlStateNormal];
    self.logoutButton.hidden = TRUE;
    [self.contentView addSubview:self.logoutButton];
    self.logoutButton.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.logoutButton.widthAnchor constraintEqualToConstant:70],
        [self.logoutButton.heightAnchor constraintEqualToConstant:26],
        [self.logoutButton.rightAnchor constraintEqualToAnchor:self.contentView.rightAnchor constant:-12],
        [self.logoutButton.centerYAnchor constraintEqualToAnchor:self.contentView.centerYAnchor constant:0]]];

    self.browseButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.browseButton setTitleColor:[UIColor colorWithRed:32.0/255.0 green:32.0/255.0 blue:32.0/255.0 alpha:1.0] forState:UIControlStateNormal];
    self.browseButton.backgroundColor = [Resources mainColor];
    self.browseButton.layer.borderColor = [UIColor clearColor].CGColor;
    self.browseButton.layer.borderWidth = 0.0f;
    self.browseButton.layer.cornerRadius = 10.0f;
    self.browseButton.titleLabel.font = [UIFont systemFontOfSize:12];
    [self.browseButton addTarget:self action:@selector(browseButtonClicked:) forControlEvents:UIControlEventTouchUpInside];
    [self.browseButton setTitle:@"browse" forState:UIControlStateNormal];
    self.browseButton.hidden = TRUE;
    [self.contentView addSubview:self.browseButton];
    self.browseButton.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.browseButton.widthAnchor constraintEqualToConstant:70],
        [self.browseButton.heightAnchor constraintEqualToConstant:26],
        [self.browseButton.rightAnchor constraintEqualToAnchor:self.logoutButton.leftAnchor constant:-4],
        [self.browseButton.centerYAnchor constraintEqualToAnchor:self.contentView.centerYAnchor constant:0]]];

    self.backgroundColor = [UIColor blackColor];

    UIView *backgroundView = [[UIView alloc] init];
    backgroundView.backgroundColor = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.15];
    self.selectedBackgroundView = backgroundView;
}

- (void)updateWithService:(YCSStreamingService *)service
{
    BOOL isLinked = service.link;
    BOOL isBroken = isLinked && service.link.isBroken;

    self.service = service;
    self.serviceName.text = service.name;
    self.brokenServiceImageView.hidden = !isBroken;
    self.username.hidden = !isLinked;
    self.username.text = isLinked ? service.link.username : nil;
    self.profileName.hidden = !isLinked;
    self.profileName.text = isLinked ? service.link.profileName : nil;
    self.browseButton.hidden = !isLinked;
    self.logoutButton.hidden = !isLinked;

    [self updateServiceNameHeight];
    [YCSConnect.shared fetchImage:service.smallThumbnailUrl cancellationTicket:nil completionHandler:^(NSData *image, NSError *error)
    {
        @try
        {
            if (error != nil)
                @throw [NSException exceptionWithName:@"fetchImageErrorException" reason:error.description userInfo:nil];

            if (image == nil)
                @throw [NSException exceptionWithName:@"nilImageException" reason:@"image data is nil" userInfo:nil];

            UIImage *img = [UIImage imageWithData:image];
            if (img == nil)
                @throw [NSException exceptionWithName:@"nilUIImageException" reason:@"UIImage is nil" userInfo:nil];

            [self.serviceImageView setImage:img];
        }
        @catch (NSException *exception)
        {
            NSLog(@"fetchImage exception - %@", exception.reason);
        }
    }];
}

- (void)updateServiceNameHeight
{
    CGFloat multiplier = 1.0;
    if (!self.username.isHidden)
    {
        multiplier = 0.5;
        if (!self.profileName.isHidden)
            multiplier = 0.33;
    }

    [self.contentView removeConstraint:self.serviceNameHeightConstraint];
    self.serviceNameHeightConstraint = [self.serviceName.heightAnchor constraintEqualToAnchor:self.serviceImageView.heightAnchor multiplier:multiplier];
    [self.contentView addConstraint:self.serviceNameHeightConstraint];
}

- (void)browseButtonClicked:(UIButton *)sender
{
    [self.actionDelegate serviceCellActionBrowse:self];
}

- (void)logoutButtonClicked:(UIButton *)sender
{
    [self.actionDelegate serviceCellActionLogout:self];
}

@end
