//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import "TokenManager.h"

@implementation TokenManager

+ (instancetype)shared
{
    static dispatch_once_t once;
    static TokenManager *instance = nil;
    dispatch_once(&once, ^{ instance = [[TokenManager alloc] init]; });
    return instance;
}

- (instancetype)init
{
    if (self = [super init])
        [self loadTokens];
	
    return self;
}

- (void)loadTokens
{
    //TODO: Replace me with keychain for improved security, but take care NOT to iCloud sync the items
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSString *accessToken = [userDefaults stringForKey:@"ycs.accessToken"];
    NSString *refreshToken = [userDefaults stringForKey:@"ycs.refreshToken"];

    UserTokens *userTokens = [[UserTokens alloc] init];
    userTokens.accessToken = accessToken;
    userTokens.refreshToken = refreshToken;
    self.userTokens = userTokens;
}

- (void)storeTokens:(NSString *)accessToken refreshToken:(NSString *)refreshToken
{
    //TODO: Replace me with keychain for improved security, but take care NOT to iCloud sync the items
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:accessToken forKey:@"ycs.accessToken"];
    [userDefaults setObject:refreshToken forKey:@"ycs.refreshToken"];
    [userDefaults synchronize];

    UserTokens *userTokens = [[UserTokens alloc] init];
    userTokens.accessToken = accessToken;
    userTokens.refreshToken = refreshToken;
    self.userTokens = userTokens;
}

#pragma mark - YCSTokenDelegate

- (void)renewTokens:(NSString *)expiredAccessToken refreshToken:(NSString *)refreshToken completionHandler:(void (^)(NSString *, NSString *))completionHandler
{
    //TODO: Implement this method to perform a backend token renewal by proxying the request
    //to your own backend servers, then to Younify using your private api key.
    @throw [NSException exceptionWithName:@"NotImplementedException" reason:@"renewTokens method has not been implemented" userInfo:nil];
}

- (void)renewedTokens:(NSString *)newAccessToken refreshToken:(NSString *)newRefreshToken
{
    //Persist any new tokens to this device
    [self storeTokens:(NSString *)newAccessToken refreshToken:(NSString *)newRefreshToken];
}

@end
